#region	 Licence and informations

/***************************************************************************************
 *		Date	:	21-4-2005
 *		Author	:	Theodoros Antoniou
 * 
 *		Description: Options for the Studio.
 *      -----------
 * 
 * 
 *	   Copyright (C) 2005  Theodoros Antoniou 
 ***************************************************************************************/
 

////////////////////////////////////////////////////////////////////////////////
//   License Agreement:                                                       //
//                                                                            //
//    This program is free software; you can redistribute it and/or modify    //
//    it under the terms of the GNU General Public License as published by    //
//    the Free Software Foundation; either version 2 of the License, or       //
//    (at your option) any later version.                                     //
//                                                                            //
//    This program is distributed in the hope that it will be useful,         //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of          //
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           //
//    GNU General Public License for more details.                            //
//                                                                            //
//    You should have received a copy of the GNU General Public License       //
//    along with this program; if not, write to the Free Software             //
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA //
//                                                                            //
//    The project web site is located on:                                     //
//      http://cdinterface.sourceforge.net									  //
//      http://www.sourceforge.net/projects/cdinterface						  //
//																			  //
//    Contact:																  //
//      defdef@users.sourceforge.net										  //
//                                                                            //                               
////////////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.IO;
using System.Collections;
using System.Windows.Forms;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;
using System.Threading;

namespace CDIStudio.Studio
{
	/// <summary>
	/// Options for the Studio.
	/// </summary>
	[Serializable]
	public class Options: ISerializable
	{
		/// <summary>
		/// Options for the Studio.
		/// </summary>
		public Options()
		{

			DefaultValues();
//			System.Diagnostics.Debug.WriteLine("   DefaultValues   " + this.CultureInfo);
		}		 

		#region	Fields
		private System.Collections.ArrayList metaData = new ArrayList();
		private string fProjectsFolder;
		private string fCultureInfo;
		private bool fAutoUpdate;
	    #endregion

		#region Propertys
		
		/// <summary>
		/// The folder where the CD-Menu projects are stored.
		/// </summary>
		public string   ProjectsFolder
		{
			get{ return	fProjectsFolder; }
			set{ fProjectsFolder = value; }
		}

		/// <summary>
		/// The folder where the CD-Menu projects are stored.
		/// </summary>
		public string   CultureInfo
		{
			get{ return	fCultureInfo; }
			set{ fCultureInfo = value; }
		}

		/// <summary>
		/// Automatically check for a new version on startup.
		/// </summary>
		public bool   AutoUpdate
		{
			get{ return	fAutoUpdate; }
			set{ fAutoUpdate = value; }
		}

		#endregion

		#region Serialization
		
		private void PopulateMetaData()
		{
			metaData.Add("ProjectsFolder");
			metaData.Add("CultureInfo");
			metaData.Add("AutoUpdate");
		}
		
		/// <summary>
		/// Deserialization constructor.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="ctxt"></param>
		public Options(SerializationInfo info, StreamingContext ctxt):this()
		{	
			try
			{
				metaData = (ArrayList)info.GetValue("MetaData",metaData.GetType());
				if(metaData.Contains("ProjectsFolder"))
					fProjectsFolder = info.GetString("ProjectsFolder");
				if(metaData.Contains("CultureInfo"))
					fCultureInfo = info.GetString("CultureInfo");
				if(metaData.Contains("AutoUpdate"))
					fAutoUpdate = info.GetBoolean("AutoUpdate");
			}
			catch
			{
				MessageBox.Show("An error with the option file occured. All settings reset to default.");
				this.DefaultValues();
			}
 		}

	    /// <summary>
	    /// Serialization function
	    /// </summary>
	    /// <param name="info"></param>
	    /// <param name="ctxt"></param>
		public void GetObjectData(SerializationInfo info, StreamingContext ctxt)
		{
			//Add MetaData
			PopulateMetaData();
			info.AddValue("MetaData", metaData);

			//Add options 
			info.AddValue("ProjectsFolder" , fProjectsFolder);
			info.AddValue("CultureInfo" , fCultureInfo);
			info.AddValue("AutoUpdate" , fAutoUpdate);
		}
	
		/// <summary>
		/// Saves it to a file.
		/// </summary>					
		public void Save()
		{			
			Stream stream = null;

			//Serialize
			try
			{
				stream = File.Open(Var.MainForm.Paths.OptionsFile, FileMode.Create);
				BinaryFormatter bformatter = new BinaryFormatter();			
				bformatter.Serialize(stream, this);
			}
			finally
			{
				if(stream != null)
					stream.Close();
			}
		}

		#endregion	



		#region Indexes


		#endregion


		#region Methods
		/// <summary>
		/// Set default values.
		/// </summary>
		public void DefaultValues()
		{
			fProjectsFolder =	System.Environment.GetFolderPath(Environment.SpecialFolder.Personal) +
								 @"\CDInterface Studio Projects";

			fCultureInfo =	Thread.CurrentThread.CurrentCulture.TwoLetterISOLanguageName;
			fAutoUpdate = true;
		}


		#endregion


		#region Events


		#endregion
	}
}
