#region	 Licence and informations

/***************************************************************************************
 *		Date	:	4-5-2005
 *		Author	:	Theodoros Antoniou
 * 
 *		Description:  Checks if a new version is available and notify the user.
 *      -----------
 * 
 * 
 *	   Copyright (C) 2005  Theodoros Antoniou 
 ***************************************************************************************/
 

////////////////////////////////////////////////////////////////////////////////
//   License Agreement:                                                       //
//                                                                            //
//    This program is free software; you can redistribute it and/or modify    //
//    it under the terms of the GNU General Public License as published by    //
//    the Free Software Foundation; either version 2 of the License, or       //
//    (at your option) any later version.                                     //
//                                                                            //
//    This program is distributed in the hope that it will be useful,         //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of          //
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           //
//    GNU General Public License for more details.                            //
//                                                                            //
//    You should have received a copy of the GNU General Public License       //
//    along with this program; if not, write to the Free Software             //
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA //
//                                                                            //
//    The project web site is located on:                                     //
//      http://cdinterface.sourceforge.net									  //
//      http://www.sourceforge.net/projects/cdinterface						  //
//																			  //
//    Contact:																  //
//      defdef@users.sourceforge.net										  //
//                                                                            //                               
////////////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Threading;
using System.Globalization;
using CustomUIControls;

namespace CDIStudio.Studio
{
	/// <summary>
	/// Notify user for new version.
	/// </summary>
	public class UpdateNotifyer : CustomUIControls.TaskbarNotifier
	{
		#region	Constructor

		public UpdateNotifyer()
		{
			CloseClickable= true;
			TitleClickable= false;
			ContentClickable= false;
			EnableSelectionRectangle= false;
			KeepVisibleOnMousOver= true;
			ReShowOnMouseOver= true;
			SetBackgroundBitmap(new Bitmap(GetType(),"UpdateNotify.png"),Color.FromArgb(255,0,255));
			SetCloseBitmap(new Bitmap(GetType(),"close.png"),Color.FromArgb(255,0,255),new Point(127,8));
			TitleRectangle=new Rectangle(55,5,70,25);
			ContentRectangle=new Rectangle(8,41,133,68);
			NormalTitleColor = Color.Blue;
			hoverTitleColor = NormalTitleColor;
			HoverTitleFont =  NormalTitleFont;

			//localize
			Var.MainForm.localization.LocaleChanged += new SimpleDelegate(this.localeChanged);
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				//Kill any thread that is running
				if(th != null)
				{
					th.Abort();
					th.Join();
				}
			}
			base.Dispose( disposing );
		}


		#endregion

		#region	Fields

		private bool VisualFeedback;
		private Thread th;
		private string CacheVersion = "";
		//Msg
		private string CheckingOutMsg;
		private string NewVersionMsg;
		private string LatestVersionMsg;
		private string ConnectToInternetMsg;
		private string UpdateMsg;

		#endregion

		#region Methods
		
		private void  GoHomePage(object sender, System.EventArgs e)
		{
			System.Diagnostics.Process.Start(@"http://cdinterface.sourceforge.net");
		}
		private void Show_Connecting()
		{
			NormalTitleColor = Color.Blue;
			EnableSelectionRectangle= false;
			ContentClickable= false;
			ContentClick -= new System.EventHandler(this.GoHomePage);
			
			Show(UpdateMsg , CheckingOutMsg ,500,3000,500);
//			Show("Update","Checking out...\n You can continue your work while the server is contacted.",500,3000,500);
		}
		private void Show_NewVersion()
		{
			NormalTitleColor = Color.Blue;
			EnableSelectionRectangle= true;
			ContentClickable= true;
			ContentClick += new System.EventHandler(this.GoHomePage);
//			Show("Update","\t New version is available.\n Do you want to download it from homepage?",500,4000,500);
			Show(UpdateMsg , NewVersionMsg ,500,4000,500);
		}
		private void Show_CurrentVersion()
		{
			NormalTitleColor = Color.Blue;
			EnableSelectionRectangle= false;
			ContentClickable= false;
			ContentClick -= new System.EventHandler(this.GoHomePage);
//			Show("Update","You have the latest version.",500,3000,500);
			Show(UpdateMsg , LatestVersionMsg ,500,4000,500);
		}
		
		private void Show_NotConnected()
		{
			NormalTitleColor = Color.Red;
			hoverTitleColor = Color.Red;
			EnableSelectionRectangle= false;
			ContentClickable= false;
			ContentClick -= new System.EventHandler(this.GoHomePage);
			Show(UpdateMsg , ConnectToInternetMsg ,500,3000,500);
		}
		private void CheckNewVersion()
		{
			try
			{
				bool connected;

				//Get the online version and store it in  "CacheVersion".
				//This way you do not have to connect to the internet every time.
				if(CacheVersion == "")
				{
					if(VisualFeedback)
					{
						Show_Connecting();
					}					
					connected = InternetCS.IsConnectedToInternet();
					if(connected)
					{

						PAD pad = new PAD();
						pad.URL =  "http://cdinterface.sourceforge.net/pad/cdinterface_studio_pad.xml";
						pad.Read();
						CacheVersion = pad.Version;
					}
					else if(VisualFeedback)
					{
						Show_NotConnected();
					}
				}

				//If you succesfull get the online version , notify the user
				//of his version.
				if(CacheVersion != "")
				{
					if(CacheVersion != Application.ProductVersion)
					{			
						Show_NewVersion();
					}
					else if(VisualFeedback)
					{
						Show_CurrentVersion();
					}
				}
			}
			catch(Exception e)
			{
				//Silent only the exception	 "ThreadAbortException"
				if(e.GetType() != typeof(ThreadAbortException))
					MessageBox.Show(e.Message);
			}
		}

		/// <summary>
		/// Checks for new version , with visual feedback to the user 
		/// that is connecting to the internet.
		/// </summary>
		public void CheckNewVersionLoud()
		{
			if(th != null)
			{
				th.Abort();
				th.Join();
			}
			VisualFeedback = true;
			th = new Thread(new ThreadStart(CheckNewVersion));
			th.Start();	
		}

		/// <summary>
		///  Checks for new version , WITHOUT visual feedback to the user
		///  that is connecting to the internet.
		/// </summary>
		public void CheckNewVersionSilent()
		{
			if(th != null)
			{
				th.Abort();
				th.Join();
			}
			VisualFeedback = false;
			th = new Thread(new ThreadStart(CheckNewVersion));
			th.Start();				
		}

		private void localeChanged()
		{
			CheckingOutMsg       =   Var.MainForm.localization["Checking out.You can continue..."];
			NewVersionMsg        =   Var.MainForm.localization["New version is available.Do you..."];
			LatestVersionMsg     =   Var.MainForm.localization["You have the latest version."];
			ConnectToInternetMsg =   Var.MainForm.localization["Please connect to the internet and try again."];
			UpdateMsg            =   Var.MainForm.localization["Update"];
		}
		#endregion
	}
}
	
