#region	 Licence and informations

/***************************************************************************************
 *		Date	:	14-3-2005
 *		Author	:	Theodoros Antoniou
 * 
 *		Description:
 *      -----------
 * 
 * 
 *	   Copyright (C) 2005  Theodoros Antoniou 
 ***************************************************************************************/
 

////////////////////////////////////////////////////////////////////////////////
//   License Agreement:                                                       //
//                                                                            //
//    This program is free software; you can redistribute it and/or modify    //
//    it under the terms of the GNU General Public License as published by    //
//    the Free Software Foundation; either version 2 of the License, or       //
//    (at your option) any later version.                                     //
//                                                                            //
//    This program is distributed in the hope that it will be useful,         //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of          //
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           //
//    GNU General Public License for more details.                            //
//                                                                            //
//    You should have received a copy of the GNU General Public License       //
//    along with this program; if not, write to the Free Software             //
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA //
//                                                                            //
//    The project web site is located on:                                     //
//      http://cdinterface.sourceforge.net									  //
//      http://www.sourceforge.net/projects/cdinterface						  //
//																			  //
//    Contact:																  //
//      defdef@users.sourceforge.net										  //
//                                                                            //                               
////////////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;
using CDIStudio.CDMenus.Forms;

namespace CDIStudio.CDMenus
{

	public enum BorderStyle : int
	{
		Fixed,
		FullScreen,
		None,
		RoundRectagle,
		Sizable
	}

	public enum FormState : int
	{
		Maximized,
		Normal
	}

	/// <summary>
	/// It is a class factory for CDMenus. Use this class to create and open CDMenus.
	/// All its members are statics so you do not have to create an instance to use it.
	/// </summary>
	public class CDMenuFactory
	{
		
		static CDMenuFactory()
		{
			cdMenusList = new Hashtable();
			RegisterCDMenu();
		}


		#region Fields
		private static CDIStudio.CDMenus.Forms.CDMenu fCDMenu;
		public static Hashtable cdMenusList;
		#endregion


		#region Propertys
		public static CDIStudio.CDMenus.Forms.CDMenu CDMenu
		{
			get
			{
				return fCDMenu;
			}
			set
			{
				fCDMenu = value;
			}
		}

		public static string CDMenuName
		{
			get
			{
				Type CDMenuType = fCDMenu.GetType();
				CDMenuInfo info = (CDMenuInfo) CDMenuType.GetProperty("Info").GetGetMethod().Invoke(fCDMenu,null);
				return info.Name;
			}
		}
		#endregion


		#region Indexes


		#endregion


		#region Methods
		/// <summary>
		/// Registered your CDMenu here to become available at the "new" menu in the main form.
		/// </summary>
		private static void RegisterCDMenu()
		{
			CDMenuFactory.RegisterCDMenu(typeof(CDAutorun));
			CDMenuFactory.RegisterCDMenu(typeof(CDSimple));
			CDMenuFactory.RegisterCDMenu(typeof(CDSingle));
//			CDMenuFactory.RegisterCDMenu(typeof(CDWebBrowser));
		}
		private static  void RegisterCDMenu(Type CDMenuType)
		{
			CDMenuInfo info = (CDMenuInfo) CDMenuType.GetProperty("Info").GetGetMethod().Invoke(null,null);  
			cdMenusList.Add(info.Name,CDMenuType);
		}	
	
		public static CDMenu CreateCDMenu(Type cdMenuType)
		{			
			CDMenu = (CDMenu)Activator.CreateInstance(cdMenuType);
			return CDMenu;	
		}

		public static CDMenu OpenCDMenu(string aFileName, bool readOnly)
		{
			//Var
			Stream stream;
			BinaryFormatter bformatter;

			//Open the file written above and read values from it.
			//readOnly allows to open the serialized file from a read only device
			//like a CD-Rom.
			if(readOnly)
			{
				stream = File.Open(aFileName, FileMode.Open, FileAccess.Read);
			}
			else
			{
				stream = File.Open(aFileName, FileMode.Open);
			}

			bformatter = new BinaryFormatter();
			
			CDMenu = (CDMenu)bformatter.Deserialize(stream);
			
			stream.Close();
			return CDMenu;
		}

		public static Type  GetCDMenu(string strClass)
		{
			return (Type)cdMenusList[strClass];
		}
	  
		public static CDMenuInfo  GetCDMenuInfo(string CDMenuName)
		{
			Type tp =	GetCDMenu(CDMenuName);
			return  (CDMenuInfo)tp.GetProperty("Info").GetGetMethod().Invoke(null,null);
		}

		public static CDMenuInfo  GetCDMenuInfo(CDIStudio.CDMenus.Forms.CDMenu CDMenuObj)
		{
			Type tp =	CDMenuObj.GetType();
			return  (CDMenuInfo)tp.GetProperty("Info").GetGetMethod().Invoke(null,null);
		}

		public static void SerializeCDMenu(string aFileName)
		{			
			Stream stream = null;

			//Serialize
			try
			{
				stream = File.Open(aFileName, FileMode.Create);
				BinaryFormatter bformatter = new BinaryFormatter();			
				bformatter.Serialize(stream, CDMenu);
			}
			finally
			{
				if(stream != null)
					stream.Close();
			}
		}
		#endregion


		#region Events


		#endregion
	}


	#region  Class CDMenuInfo
	/// <summary>
	/// Static informations for CDMenus.
	/// </summary>
	public class CDMenuInfo 
	{
		public CDMenuInfo()
		{
		}

		#region Fields
		private string fName;
		private string fDescription;
		private string[] fAuthors;
		private string[] fLibrarys;
		#endregion


		#region Propertys
		/// <summary>
		/// The name that will be displayed in the new dialog of CDI.
		/// Also this is used by the CDMenuFactory to register and create the CDMenu.
		/// </summary>
		public string Name
		{
			get{ return fName; }
			set{ fName = value; }
		}
	
		/// <summary>
		/// A short description what this CDMenu do.
		/// </summary>
		public string Description
		{
			get{ return fDescription; }
			set{ fDescription = value; }
		}

		/// <summary>
		/// The people who wrote this CDMenu.
		/// </summary>
		public string[] Authors
		{
			get{ return fAuthors; }
			set{ fAuthors = value; }
		}

		/// <summary>
		/// Dependencies of this CDMenu. Not standard Librarys that are needed. CDI copy this
		/// librarys in the "BinFolder" of the project.
		/// </summary>
		public string[] Librarys
		{
			get{ return fLibrarys; }
			set{ fLibrarys = value; }
		}
		#endregion


		#region Indexes


		#endregion


		#region Methods


		#endregion
	}
	#endregion

	#region	Class CDOptions

	/// <summary>
	/// Options for CDMenus.
	/// </summary>
	public class CDOptions
	{
		public CDOptions()
		{
			fAutorun = new ActionParameters();
		}


		private CDIStudio.CDMenus.ActionParameters fAutorun;
		private System.Drawing.Icon fIcon;

		/// <summary>
		/// Action executed when the CDMenu run.
		/// </summary>
		public CDIStudio.CDMenus.ActionParameters Autorun
		{
			get{ return fAutorun; }
			set{ fAutorun = value; }			
		}

		/// <summary>
		/// The icon that is displayed on the CD. CDI is copyed in the "CDMenuFolder" 
		/// of the project.
		/// </summary>
		public System.Drawing.Icon Icon
		{
			get{ return fIcon; }
			set { fIcon = value; }
		}



	}

	#endregion
	
}
