
#region	 Licence and informations



/***************************************************************************************
 *		Date	:	15-4-2005
 *		Author	:	Lucian Rosu
 * 
 *		Description:
 *      -----------
 * 
 * 
 *	   Copyright (C) 2005  Theodoros Antoniou 
 ***************************************************************************************/
 

////////////////////////////////////////////////////////////////////////////////
//   License Agreement:                                                       //
//                                                                            //
//    This program is free software; you can redistribute it and/or modify    //
//    it under the terms of the GNU General Public License as published by    //
//    the Free Software Foundation; either version 2 of the License, or       //
//    (at your option) any later version.                                     //
//                                                                            //
//    This program is distributed in the hope that it will be useful,         //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of          //
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           //
//    GNU General Public License for more details.                            //
//                                                                            //
//    You should have received a copy of the GNU General Public License       //
//    along with this program; if not, write to the Free Software             //
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA //
//                                                                            //
//    The project web site is located on:                                     //
//      http://www.sourceforge.net/projects/cdinterface.                      //
//      http://www.cdinterface.sourceforge.net                                //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using CDIStudio.CDMenus;
using System.Windows.Forms;
using Finisar.SQLite;
using System.Data;

namespace CDIStudio.CDMenus.DB
{
	/// <summary>
	/// Summary description for DB.
	/// </summary>
	public class CustomDB : CDIStudio.CDMenus.DB.DataBase
	{
		public CustomDB()
		{


			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}




		#region Fields
		

		#endregion


		#region Propertys
		private string SQLCreateString
		{
			get
			{
				string str = @"
					CREATE TABLE actions (
					id INTEGER PRIMARY KEY UNIQUE, 
					software_title VARCHAR NOT NULL DEFAULT 'title' , 
					parameter1 VARCHAR , 
					parameter2 VARCHAR , 
					parameter3 VARCHAR , 
					parameter4 VARCHAR , 
					parameter5 VARCHAR ,
					info VARCHAR);

					CREATE INDEX title_index ON actions (software_title);
				";

				return str;
			}
		}

		public	DataTable actionsDT
		{
			get{ return DS.Tables["actions"];	}
		}

		#endregion


		#region Indexes


		#endregion


		#region Methods

		/// <summary>
		/// Override this method to create a new database.
		/// </summary>
		public override void CreateDB()
		{
			try
			{
				//Create connection
				connection = new SQLiteConnection();
				connection.ConnectionString = ConnectionStringNew;
				connection.Open();

				//Create db
 				command = connection.CreateCommand();
				command.CommandText = SQLCreateString;
				command.ExecuteNonQuery();
			}
			catch (SQLiteException e)
			{
				MessageBox.Show(e.Message);
			}
			finally
			{
				connection.Close();
			}				
		}

		/// <summary>
		/// Override this method to open the database.
		/// </summary>
		public override void OpenDB()
		{
			try
			{
				//Create connection
				connection = new SQLiteConnection();
				connection.ConnectionString = ConnectionStringOpen;

				//fill dataset
				selectStr = @"select * from actions order by software_title;";
				dataAdapter = new SQLiteDataAdapter(selectStr,connection);
				fDS = new DataSet();
				dataAdapter.Fill(fDS);
				fDS.Tables[0].TableName = "actions";

				//Creating modifycation	commands
				createInsertCommand();
				createUpdateCommand();
				createDeleteCommand();
			}
			catch (SQLiteException e)
			{
				MessageBox.Show(e.Message);
			}
			finally
			{
				connection.Close();
			}				
		}

		/// <summary>
		/// Override this method to close the database.
		/// </summary>
		//public override void CloseDB(){}

		/// <summary>
		/// Override this method to save the database.
		/// </summary>
		public override void SaveDB()
		{	
			try
			{
				dataAdapter.Update(DS,"actions");
			}
			catch(SQLiteException ex)
			{
				MessageBox.Show(ex.Message);
			}
		}


		/// <summary>
		/// Override this method to create the DataPanel.
		/// </summary>
		public override void CreateDataPanel()
		{
			fDataPanel = new CustomDataPanel(this);
		}


		private void createInsertCommand()
		{
			InsertStr =@" 
				INSERT INTO actions
				(
					software_title, 
					parameter1, 
					parameter2, 
					parameter3, 
					parameter4, 
					parameter5,
					info
				)
				VALUES
				(
					@software_title, 
					@parameter1, 
					@parameter2, 
					@parameter3, 
					@parameter4, 
					@parameter5,
					@info
				);							
			";

			//Create command
			command = connection.CreateCommand();
			command.CommandText = InsertStr;
			dataAdapter.InsertCommand = command;

			//Create parameters
			command.Parameters.Add(
				"@software_title",
				DbType.AnsiString,
				80,
				"software_title");

			command.Parameters.Add(
				"@parameter1",
				DbType.AnsiString,
				15,
				"parameter1");

			command.Parameters.Add(
				"@parameter2",
				DbType.AnsiString,
				100,
				"parameter2");

			command.Parameters.Add(
				"@parameter3",
				DbType.AnsiString,
				100,
				"parameter3");

			command.Parameters.Add(
				"@parameter4",
				DbType.AnsiString,
				100,
				"parameter4");

			command.Parameters.Add(
				"@parameter5",
				DbType.AnsiString,
				100,
				"parameter5");
		 
			command.Parameters.Add(
				"@info",
				DbType.AnsiString,
				500,
				"info");
		}
		private void createUpdateCommand()
		{
			UpdateStr =@" 
				UPDATE actions
				SET
					software_title = @software_title, 
					parameter1 = @parameter1, 
					parameter2 = @parameter2, 
					parameter3 = @parameter3, 
					parameter4 = @parameter4, 
					parameter5 = @parameter5,
					info = @info
				WHERE
					id = @id;											
			";

			//Create command
			command = connection.CreateCommand();
			command.CommandText = UpdateStr;
			dataAdapter.UpdateCommand = command;

			//Create parameters
			Finisar.SQLite.SQLiteParameter parm =
			command.Parameters.Add(
				"@id",
				DbType.Int32,
				4,
				"id");
			parm.SourceVersion = DataRowVersion.Original;

			command.Parameters.Add(
				"@software_title",
				DbType.AnsiString,
				80,
				"software_title");

			command.Parameters.Add(
				"@parameter1",
				DbType.AnsiString,
				15,
				"parameter1");

			command.Parameters.Add(
				"@parameter2",
				DbType.AnsiString,
				100,
				"parameter2");

			command.Parameters.Add(
				"@parameter3",
				DbType.AnsiString,
				100,
				"parameter3");

			command.Parameters.Add(
				"@parameter4",
				DbType.AnsiString,
				100,
				"parameter4");

			command.Parameters.Add(
				"@parameter5",
				DbType.AnsiString,
				100,
				"parameter5");

			command.Parameters.Add(
				"@info",
				DbType.AnsiString,
				500,
				"info");
		}
		private void createDeleteCommand()
		{
			DeleteStr =@" 
				DELETE FROM actions
				WHERE
					id = @id;							
			";

			//Create command
			command = connection.CreateCommand();
			command.CommandText = DeleteStr;
			dataAdapter.DeleteCommand = command;

			//Create parameters
			command.Parameters.Add(
				"@id",
				DbType.Int32,
				4,
				"id");
		}

		#endregion

	}
}
