#region	 Licence and informations



/***************************************************************************************
 *		Date	:	14-3-2005
 *		Author	:	Theodoros Antoniou
 * 
 *		Description:
 *      -----------
 * 
 * 
 *	   Copyright (C) 2005  Theodoros Antoniou 
 ***************************************************************************************/
 

////////////////////////////////////////////////////////////////////////////////
//   License Agreement:                                                       //
//                                                                            //
//    This program is free software; you can redistribute it and/or modify    //
//    it under the terms of the GNU General Public License as published by    //
//    the Free Software Foundation; either version 2 of the License, or       //
//    (at your option) any later version.                                     //
//                                                                            //
//    This program is distributed in the hope that it will be useful,         //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of          //
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           //
//    GNU General Public License for more details.                            //
//                                                                            //
//    You should have received a copy of the GNU General Public License       //
//    along with this program; if not, write to the Free Software             //
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA //
//                                                                            //
//    The project web site is located on:                                     //
//      http://cdinterface.sourceforge.net									  //
//      http://www.sourceforge.net/projects/cdinterface						  //
//																			  //
//    Contact:																  //
//      defdef@users.sourceforge.net										  //
//                                                                            //                               
////////////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using CDIStudio.CDMenus.DB;
using CDIStudio.CDMenus.PropertyTemplates;

namespace CDIStudio.CDMenus.Forms
{

	#region	 Class CDMenu
	    
	/// <summary>
	/// CDMenu is the base class of all 'CDMenus'.
	/// </summary>
	[Serializable()] 
	public class CDMenu : System.Windows.Forms.Form, ISerializable
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// The base class of all CD-Menu.
		/// </summary>
		public CDMenu()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			InitializeCDMenu();			 
		}

		static CDMenu()
		{
			fInfo = new CDMenuInfo();
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
				

				if( fDB != null)
				{
					if( fDB.dataPanel != null)
					{
						fDB.dataPanel.Dispose();
					}
					fDB = null;
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(CDMenu));
			// 
			// CDMenu
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Window;
			this.ClientSize = new System.Drawing.Size(336, 278);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Name = "CDMenu";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "CDMenu";
			this.Load += new System.EventHandler(this.CDMenu_Load);

		}
		#endregion


		#region Version


		/// <summary>
		/// Major version of this CDMenu.
		/// </summary>
		protected int fMajorVersion = 1;

		/// <summary>
		/// Minor version of this CDMenu.
		/// </summary>
		protected int fMinorVersion = 0;

		/// <summary>
		/// Major version of this CD-Menu
		/// </summary>
		public int MajorVersion { get { return fMajorVersion; } }

		/// <summary>
		/// Minor version of this CD-Menu
		/// </summary>
		public int MinorVersion { get { return fMinorVersion; } }

		/// <summary>
		/// String representation of version.
		/// </summary>
		public string  CDInfoVersion	
		{ 
			get
			{
				return	Convert.ToString(MajorVersion) + "." +  Convert.ToString(MinorVersion); 
			} 
		}


		#endregion

		#region	CDMenu Info

		private static CDMenuInfo fInfo;

		/// <summary>
		/// Informations about this CD-Menu (CDName ,Description , Authors)
		/// </summary>
		static public CDMenuInfo Info
		{
			get{ return Info;   }
		}

		#endregion

		#region	CD

		CDOptions fOptions;

		public CDOptions Options
		{
			get { return fOptions; }
		}

		#endregion

 		#region	 Directorys

		private static string fProjectFolder;

		public static string ProjectFolder
		{
			get
			{
				return fProjectFolder;
			}
			set
			{
				fProjectFolder = value;
			}
		}

		public static string MyFilesFolder
		{
			get
			{
				return fProjectFolder + @"\My Files";
			}
		}
		public static string DataFolder
		{
			get
			{
				return fProjectFolder + @"\Data";;
			}
		}
		public static string BinFolder
		{
			get
			{
				return DataFolder + @"\Bin";;
			}
		}

		public static string CDMenuFolder
		{
			get
			{
				return DataFolder + @"\CDMenu";;
			}
		}
		public static string DBFolder
		{
			get
			{
				return DataFolder + @"\DB";;
			}
		}


		public static string GetFullPath(string aFileName)
		{
			if(	ProjectFolder.ToLower() == aFileName.ToLower())
			{	return ProjectFolder;	}
			else
			{
				//return Path.GetFullPath(aFileName);
				aFileName = ProjectFolder + @"\" + aFileName;
				return aFileName;
			}
		}

		public static string GetRelativePath(string aFileName)
		{			
			if(	ProjectFolder.ToLower() == aFileName.ToLower())
			{	return "";	}
			else
			{
				string aProjectFolder =	 ProjectFolder + @"\";
				aFileName = aFileName.Remove(0,aProjectFolder.Length);
				return aFileName;
			}
		}
		#endregion

		#region	 Serialization and PropertyEditor support

		//Fields
		private CDIStudio.CDMenus.PropertyBag fPropertyBag;
		private PropertyTemplateManager fPropManager;

		
		public CDIStudio.CDMenus.PropertyBag PropBag
		{
			get
			{
				return fPropertyBag;
			}

			set
			{
				fPropertyBag = value;
			}
		}

		public  PropertyTemplateManager PropManager
		{
			get
			{
				return fPropManager;
			}
			set
			{
				fPropManager = value;
			}
		}

		private void propertysGetValue(object sender, PropertySpecEventArgs e)
		{
			PropManager.PropertysGetValue(e);

		}
		private void propertysSetValue(object sender, PropertySpecEventArgs e)
		{
			PropManager.PropertysSetValue(e);

		}	

		//Serialization
		public virtual void GetObjectData(SerializationInfo info, StreamingContext ctxt){}
		
	    #endregion

		#region ProcessManager

		public static int MaxAutorunParam
		{get {return 2;}}
		
		private CDIStudio.CDMenus.ProcessManager fProcessManager;
		

		public CDIStudio.CDMenus.ProcessManager	ProcessManager
		{
			get
			{
				return fProcessManager;
			}

			set
			{
				fProcessManager = value;
			}
		}



		#endregion

		#region Initializations

		/// <summary>
		/// It is called  at construction time.
		/// </summary>
		private void InitializeCDMenu()
		{			
			fOptions = new CDOptions();
			fPropManager = new PropertyTemplateManager(this);
			this.fPropertyBag = new PropertyBag();
			fPropertyBag.GetValue += new PropertySpecEventHandler(this.propertysGetValue);
			fPropertyBag.SetValue += new PropertySpecEventHandler(this.propertysSetValue);
		

			fProcessManager = new ProcessManager();
					
		}


		private void CDMenu_Load(object sender, System.EventArgs e)
		{
			if(!DesignMode)
			{
				PropManager.AddPropertys();
			}			
		}

		
		/// <summary>
		/// It is called when the CDMenu is instantiate at Studio.
		/// </summary>
		public void  StudioInitialization()
		{
				
		}


		/// <summary>
		/// It is called when the CDMenu is instantiate at autorun.
		/// </summary>
		public virtual void  AutorunInitialization()
		{
			
		}


		#endregion		
		
		#region	Database

		protected CDIStudio.CDMenus.DB.DataBase fDB;

		public CDIStudio.CDMenus.DB.DataBase DB
		{
			get{ return fDB;}
		}

		public virtual void InitializeDataBindings(){}
		
		#endregion

		#region	Design
		/// <summary>
		/// Gives a round rectagle look to the client area of the form.
		/// Also set the "FormBorderStyle" to "None".
		/// </summary>
		public void CreateRoundRectagle()
		{
			FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;

			//Create  point array that holds a round rectagle
			int w = this.ClientRectangle.Width;
			int	h = this.ClientRectangle.Height;

			Point[] pt = new Point[12];

			pt[0] = new Point(0,6);
			pt[1] = new Point(2,2);
			pt[2] = new Point(6,0);

			pt[3] = new Point(w-6,0);
			pt[4] = new Point(w-2,2);
			pt[5] = new Point(w,6);

			pt[6] = new Point(w,h-6);
			pt[7] = new Point(w-2,h-2);
			pt[8] = new Point(w-6,h);

			pt[9] = new Point(6,h);
			pt[10] = new Point(2,h-2);
			pt[11] = new Point(0,h-6);

			//Create region
			System.Drawing.Drawing2D.GraphicsPath reg = new System.Drawing.Drawing2D.GraphicsPath();
			reg.AddPolygon(pt);
			this.Region = new Region(reg);
			this.MaximizeBox = true;
			this.ControlBox = true;
		}


		#region	ClientMoveable
		private bool fClientMoveable;
		private Point mouseOffset;
		private bool isMouseDown = false;
		private Control CtrlMove;

		public bool ClientMoveable
		{
			get
			{
				return fClientMoveable;
			}
			set
			{
				fClientMoveable = value;
			}
		}
		protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs e)
		{
			if(ClientMoveable)
			{
				if (isMouseDown) 
				{			
					Point mousePos = Control.MousePosition;
					mousePos.Offset(mouseOffset.X, mouseOffset.Y);
					this.FindForm().Location = mousePos;  
					
				}
			}
		}

		protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			if(ClientMoveable)
			{
				int xOffset;
				int yOffset;
				   
				if (e.Button == MouseButtons.Left) 
				{   
					xOffset = -e.X ;
					yOffset = -e.Y ; 
					mouseOffset =  CtrlMove.PointToClient(this.PointToScreen(new Point(xOffset, yOffset)));
					isMouseDown = true;
				}
			}
		}

		protected override void OnMouseUp( System.Windows.Forms.MouseEventArgs e)
		{
			if(ClientMoveable)
			{
				// Changes the isMouseDown field so that the form does
				// not move unless the user is pressing the left mouse button.
				if (e.Button == MouseButtons.Left) 
				{
					isMouseDown = false;
				}
			}
		}


		/// <summary>
		/// Add the controls which you want to act like a title bar , to be able to move the form.
		/// </summary>
		/// <param name="Ctrls"></param>
		protected void MoveableControls(Control[] Ctrls)
		{
			foreach(Control Ctrl in Ctrls)
			{
				Ctrl.MouseMove += new System.Windows.Forms.MouseEventHandler(this.ControlMovable_MouseMove);
				Ctrl.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ControlMovable_MouseDown);
			    Ctrl.MouseUp += new System.Windows.Forms.MouseEventHandler(this.ControlMovable_MouseUp);
			}
		}

		private void ControlMovable_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			this.OnMouseMove(e);
		}

		private void ControlMovable_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			CtrlMove = (Control)sender;
			this.OnMouseDown(e);
		}

		private void ControlMovable_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			this.OnMouseUp(e);
		}
		#endregion
		
		#endregion

		#region	 Propertys

		PropertyGrid fPropertyGrid;
		public PropertyGrid PropertyGrid
		{
			get
			{
				return fPropertyGrid;
			}
			set
			{
				fPropertyGrid = value;
			}
		}


		CDIStudio.CDMenus.BorderStyle	 borderStyle;
		public   CDIStudio.CDMenus.BorderStyle BorderStyle
		{
			get
			{
				return borderStyle;
			}
			set
			{
				borderStyle = value;
			}
		}


		CDIStudio.CDMenus.FormState	 formState;
		public   CDIStudio.CDMenus.FormState FormState
		{
			get
			{
				return formState;
			}
			set
			{
				formState = value;
				if(IDEMode)
					this.WindowState = System.Windows.Forms.FormWindowState.Normal;
				else if(formState == CDIStudio.CDMenus.FormState.Maximized)
					this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
				else if(formState == CDIStudio.CDMenus.FormState.Normal)
					this.WindowState = System.Windows.Forms.FormWindowState.Normal;
			}
		}

		
		bool ideMode = false;
		/// <summary>
		/// A flag that shows if the cd-menu is open inside CDI IDE
		/// for editing.
		/// </summary>
		public bool IDEMode 
		{
			get
			{
				return ideMode;
			}
			set
			{
				ideMode = value;
			}
		}

		#endregion
		
		//****************************  Nested Classes  **************************************


		#region	 Class PropertyTemplateManager
		public class PropertyTemplateManager
		{	
	
			#region	 Constructor
    
			public PropertyTemplateManager(CDMenu aCDMenu)
			{
				if(aCDMenu != null)
				cdMenu = aCDMenu;
				fMetaData = new ArrayList();
				AddTemplates(new PropTemplateCDOptions(cdMenu.Options, "Options"));
			}

			#endregion

			#region	 Fields	& propertys
	    
			CDMenu cdMenu;
			System.Collections.ArrayList fItems = new ArrayList();
			System.Collections.ArrayList fMetaData = new ArrayList();

			public	 System.Collections.ArrayList Items
			{
				get{	return fItems;	}
				set{	fItems = value; }
			}

			public	 System.Collections.ArrayList MetaData
			{
				get{	return fMetaData;	}
			}

			#endregion

			#region	Public Methods
	    
			public void AddTemplates(PropertyTemplates.PropTemplate template)
			{
				fItems.Add(template);
			}

			public void AddPropertys()
			{
				//Items
				foreach(PropertyTemplates.PropTemplate templ in fItems)
				{
					templ.AddPropertys();
				}	
			}
			public void PropertysGetValue(PropertySpecEventArgs e)
			{
				//Items
				foreach(PropertyTemplates.PropTemplate templ in fItems)
				{
					if (e.Property.Category == templ.Category)
						templ.PropertysGetValue(e);
				}
			}
			public void PropertysSetValue(PropertySpecEventArgs e)
			{
				//Items
				foreach(PropertyTemplates.PropTemplate templ in fItems)
				{
					if (e.Property.Category == templ.Category)
						templ.PropertysSetValue(e);
				}
			}		
			public void Serialize(SerializationInfo info)
			{
				MetaData.Clear();
				//Items
				foreach(PropertyTemplates.PropTemplate templ in fItems)
				{
					templ.Serialize(info, MetaData);
				}

				info.AddValue("MetaData", MetaData);			 
			}

			public void Deserialize(SerializationInfo info)
			{
				fMetaData = (System.Collections.ArrayList) info.GetValue("MetaData",typeof(System.Collections.ArrayList));
	
				//Items
				foreach(PropertyTemplates.PropTemplate templ in fItems)
				{
					templ.Deserialize(info , MetaData);
				}
			}

			#endregion

		}

		#endregion


	}

	#endregion

}
