#region	 Licence and informations



/***************************************************************************************
 *		Date	:	14-3-2005
 *		Author	:	Theodoros Antoniou
 * 
 *		Description:
 *      -----------
 * 
 * 
 *	   Copyright (C) 2005  Theodoros Antoniou 
 ***************************************************************************************/
 

////////////////////////////////////////////////////////////////////////////////
//   License Agreement:                                                       //
//                                                                            //
//    This program is free software; you can redistribute it and/or modify    //
//    it under the terms of the GNU General Public License as published by    //
//    the Free Software Foundation; either version 2 of the License, or       //
//    (at your option) any later version.                                     //
//                                                                            //
//    This program is distributed in the hope that it will be useful,         //
//    but WITHOUT ANY WARRANTY; without even the implied warranty of          //
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           //
//    GNU General Public License for more details.                            //
//                                                                            //
//    You should have received a copy of the GNU General Public License       //
//    along with this program; if not, write to the Free Software             //
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA //
//                                                                            //
//    The project web site is located on:                                     //
//      http://cdinterface.sourceforge.net									  //
//      http://www.sourceforge.net/projects/cdinterface						  //
//																			  //
//    Contact:																  //
//      defdef@users.sourceforge.net										  //
//                                                                            //                               
////////////////////////////////////////////////////////////////////////////////
#endregion

using System;
using System.Diagnostics;
using System.IO;
using System.Runtime.Serialization;
using System.ComponentModel;
using System.Globalization;
using System.Drawing;
using CDIStudio.CDMenus.Forms;
using System.Windows.Forms;


namespace CDIStudio.CDMenus
{
	#region	 ProcessManager

	/// <summary>
	/// ProcessManager use "Process" class to start new process. The process are
	/// relative to "CDMenu.ProjectFolder" where necessary.
	/// </summary>
	public  class ProcessManager
	{
		public  ProcessManager()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		#region Methods

		public static void Start(CDIStudio.CDMenus.ActionParameters param)
		{
			try
			{

				switch(param[0])
				{
					case "None" :
						break;
					case "File" :
					{
						if (param[1] == "")
						{	break;	}
						System.Diagnostics.Process.Start(CDMenu.GetFullPath(param[1]));
					}
						break;
					case "Folder" :
					{
						System.Diagnostics.Process.Start(CDMenu.GetFullPath(param[1]));
					}
						break;
					case "Internet" :
					{
						if (param[1] == "")
						{	break;	}
						System.Diagnostics.Process.Start(@"http://" + param[1]);
					}
						break;
					case "E-mail" :
					{
						System.Diagnostics.Process.Start(param[1]);
					}
						break;
					default :
						break;
				}
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message);
			}
		}


		#endregion

		
	}

	#endregion


	#region	 ActionParameters	    

	/// <summary>
	/// Holds all the information for a process.
	/// </summary>
	[Serializable]
	public sealed class ActionParameters  //:  System.ICloneable
	{
		private string fName;
		public int Count = 5;
		public string[] Parameters;

		public ActionParameters()
		{
			Parameters =  new string[Count];
			Parameters[0] = "None";
		}

		/// <summary>
		/// Deserialization constructor.
		/// </summary>
		/// <param name="info"></param>
		/// <param name="ctxt"></param>
		public ActionParameters(SerializationInfo info, StreamingContext ctxt):this()
		{	
			//this.Autorun = new ActionParameters();
			this.Parameters = (string[])(info.GetValue("Parameters", typeof(string[])));

		}

		//Copy constructor
		public ActionParameters Clone()
		{
			ActionParameters clone;
			clone = new ActionParameters();

			for(int i=0; i<Count; i++)
			{	clone[i] = this.Parameters[i]; }
			return	clone;

		}

		public string this[int index]
		{
			get{ return Parameters[index];}
			set{ Parameters[index] = value;}
		}
		

		public void GetObjectData(SerializationInfo info, StreamingContext ctxt)
		{
			//You can use any custom name for your name-value pair. But make sure you
			// read the values with the same name. For ex:- If you write EmpId as "EmployeeId"
			// then you should read the same with "EmployeeId"
			//info.AddValue("EmployeeName", this);

			//Autorun parameters
			info.AddValue("Parameters", this.Parameters);			
		}

		public string Name
		{
			get{ return fName; }
			set{ fName = value; }
		}

		public void CopyParameters(ActionParameters actParam)
		{
			for(int i=0 ; i < this.Count-1 ; i++)
			{
				actParam[i] =this[i];
			}
		}
	}

	 #endregion


	#region	ActionParametersConverter

 	/// <summary>
	/// ConverterType for ActionParametersConverter.
	/// </summary>
	public class ActionParametersConverter : TypeConverter 
	{
		// Overrides the CanConvertFrom method of TypeConverter.
		// The ITypeDescriptorContext interface provides the context for the
		// conversion. Typically, this interface is used at design time to 
		// provide information about the design-time container.
		public override bool CanConvertFrom(ITypeDescriptorContext context, 
			Type sourceType) 
		{
			
//			if (sourceType == typeof(string)) 
//			{
//				return false;
//			}
			return base.CanConvertFrom(context, sourceType);
		}

		// Overrides the ConvertFrom method of TypeConverter.
		public override object ConvertFrom(ITypeDescriptorContext context, 
			CultureInfo culture, object value) 
		{
			//
			
			if (value is string) 
			{
				string[] v = ((string)value).Split(new char[] {':'});
				ActionParameters ap = new ActionParameters();
				ap[0] = v[0];
			    ap[1] = v[1];
				return ap;

			}											   
			return base.ConvertFrom(context, culture, value);	
		}
		// Overrides the ConvertTo method of TypeConverter.
		public override object ConvertTo(ITypeDescriptorContext context, 
			CultureInfo culture, object value, Type destinationType) 
		{  
			if (destinationType == typeof(string)) 
			{
				//return ((Point)value).X + "," + ((Point)value).Y;
				return ((ActionParameters)value).Parameters[0] + @" : """ +
					   ((ActionParameters)value).Parameters[1] + @"""";
			}
			return base.ConvertTo(context, culture, value, destinationType);
		}
	}

	#endregion

}
